project.EpisodeDetails = function(dom_element, options)
{
    this.dom_element = dom_element;

    this.logging_prefix = 'EpisodeDetails';

    var that = this;

    this.col2 = this.dom_element.find('#col2_content');

    this.subnav_element = this.dom_element.find('.subnav').first();

    this.col1_element = jQuery('#col1');
    this.col2_element = jQuery('#col2');

    // Get params from URL-Hash
    this.show_id = null;
    this.origin = jQuery.getUrlVar('origin') || 'shows';

    this.episode_id = jQuery.getUrlVar('episodeid') || '';

    if (this.episode_id === '' || this.episode_id === 'undefined')
    {
        this.dom_element.find('.js_ed_showtitle').text('Episode not found. Please try another one. :-)');
        this.col1_element.hide();
        this.col2_element.hide();
        this.subnav_element.hide();
        return;
    }

    project.data_provider.callWebApi("find_episode&episodeid=" + this.episode_id, function(episode)
    {
        that.trace('episode title: ' + episode.title);
        that.trace('video_id: ' + episode.video_id);

        project.track(episode.title, 'Shows', episode.show_title, episode.title);

        if (!episode.title || !episode.video_id)
        {
            this.dom_element.find('.js_ed_showtitle').text('Episode not found. Please try another one. :-)');
            this.col1_element.hide();
            this.col2_element.hide();
            this.subnav_element.hide();
            return;
        }

        that.show_id = episode.show_id;
        that.dom_element.find('.js_ed_showtitle').text(episode.show_title);
        that.col2.find('.js_ed_title').html(episode.episode_number + ': ' + episode.title);
        that.col2.find('.js_ed_text').html(project.string_toolkit.createCleanExcerpt(episode.text, 350));

        if (episode.video_url && episode.video_url.FLVURL)
        {
            that.trace('directly starting video w/ URL we got from find_episode call', episode.video_url.FLVURL);
            that.player_url = episode.video_url.FLVURL;
        }
        else
        {
            that.trace('asking api for url of video as we did not get it via find_episode');
            var self = that;
            project.data_provider.callWebApi("get_video_url&video_id=" + episode.video_id, function(data)
            {
                self.trace('get_video_url callback', data.FLVURL);
                self.player_url = data.FLVURL;
            });
        }

        that.player = new project.Player({
            'url': that.player_url,
            'size': {
                'start_x': 42,
                'start_y': 155,
                'screen_width': 284,
                'screen_height': 524
            },
            'tracking_title': episode.title,
            'tracking_section': 'Shows',
            'tracking_subsection1': episode.show_title,
            'tracking_subsection2': episode.title
        });

        that.player.play();

        if (episode.prev_episode !== "null")
        {
            that.subnav_element.find(".js_previous_episode").click(function(e)
            {
                e.preventDefault();
                project.navigateTo("episode_detail.html?episodeid=" + episode.prev_episode);
            });
        }
        else
        {
            that.subnav_element.find(".js_previous_episode").hide();
        }

        if (episode.next_episode !== "null")
        {
            that.subnav_element.find(".js_next_episode").click(function(e)
            {
                e.preventDefault();
                project.navigateTo("episode_detail.html?episodeid=" + episode.next_episode);
            });
        }
        else
        {
            that.subnav_element.find(".js_next_episode").hide();
        }

        if (episode.show_id !== "null")
        {
            that.subnav_element.find(".js_goto_show").click(function(e)
            {
                e.preventDefault();
                project.navigateTo("episodes.html?showid=" + episode.show_id + "#subpage=all_episodes");
            });                    
        }
        else
        {
            that.subnav_element.find(".js_goto_show").hide();
        }

        that.subnav_element.show();
    });

    this.initializeNavigation();
};

project.EpisodeDetails.prototype = {

    initializeNavigation: function()
    {
        var that = this;

        this.key_controller = new gtv.jq.KeyController();

        var nav_key_mapping = {};

        var PAGE_UP = 33;
        var PAGE_DOWN = 34;

        nav_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('selected', jQuery('.pc_fullscreen a'));
        };

        nav_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            var newSelected = jQuery('.subnav .js_item:visible').first();

            if (newSelected.length < 1)
            {
                newSelected = jQuery('.pc_fullscreen a');
            }

            return new gtv.jq.Selection('selected', newSelected);
        };

        var subnav_key_mapping = {};

        subnav_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('selected', jQuery('.pc_fullscreen a'));
        };
        subnav_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            selectedItem.click();
            return new gtv.jq.Selection('selected', selectedItem);
        };

        var player_key_mapping = {};
        player_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            var parent = selectedItem.parent();

            if (that.player && parent.hasClass('pc_fullscreen'))
            {
                that.player.toggleFullscreenMode();
                selectedItem = jQuery('#player_controls').find('.js_item').first();
            }
            else if (that.player && parent.hasClass('pc_pause'))
            {
                if (that.player.pause())
                {
                    selectedItem = jQuery('.pc_play a');
                }
            }
            else if (that.player && parent.hasClass('pc_stop'))
            {
                that.player.stop();
            }
            else if (that.player && parent.hasClass('pc_play'))
            {
                that.player.play();
                selectedItem = jQuery('.pc_pause a');
            }
            else if (that.player && parent.hasClass('pc_bw'))
            {
                that.player.rewind();
            }
            else if (that.player && parent.hasClass('pc_fw'))
            {
                that.player.forward();
            }

            return new gtv.jq.Selection('selected', selectedItem);
        };

        player_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            if (selectedItem.is(jQuery('#player_controls .js_item').first()))
            {
                return new gtv.jq.Selection('selected', jQuery('#player_controls .js_item').last());
            }
            return new gtv.jq.Selection('none');
        };

        player_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            if (that.player.fullscreen)
            {
                return new gtv.jq.Selection('skip');
            }

            var newSelected = jQuery('.subnav .js_item:visible').first();

            if (newSelected.length < 1)
            {
                newSelected = jQuery('#nav .js_item').last();
            }

            return new gtv.jq.Selection('selected', newSelected);
        };

        player_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            if (selectedItem.is(jQuery('#player_controls .js_item').last()))
            {
                return new gtv.jq.Selection('selected', jQuery('#player_controls .js_item').first());
            }
            return new gtv.jq.Selection('none');
        };

        player_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            if (that.player.fullscreen)
            {
                return new gtv.jq.Selection('skip');
            }

            return new gtv.jq.Selection('selected', jQuery('#nav .js_item').first());
        };

        this.nav_zone = project.getMenuBehaviourZone(nav_key_mapping);
        this.subnav_zone = project.getSubMenuBehaviourZone(subnav_key_mapping);
        this.player_zone = project.getDefaultBehaviourZone('#player_controls', player_key_mapping);

        this.key_controller.addBehaviorZone(this.nav_zone);
        this.key_controller.addBehaviorZone(this.subnav_zone);
        this.key_controller.addBehaviorZone(this.player_zone);

        var global_mapping = {};
        global_mapping[project.keycodes.KEY_PLAY] = function(selectedItem, newSelected)
        {
            that.player.play();
            return new gtv.jq.Selection('selected', jQuery('.pc_pause a'));
        };

        global_mapping[project.keycodes.KEY_STOP] = function(selectedItem, newSelected)
        {
            if (that.player.stop())
            {
                return new gtv.jq.Selection('selected', jQuery('.pc_stop a'));
            }
            return new gtv.jq.Selection('skip');
        };

        global_mapping[project.keycodes.KEY_PAUSE] = function(selectedItem, newSelected)
        {
            if (that.player.pause())
            {
                return new gtv.jq.Selection('selected', jQuery('.pc_play a'));
            }
            return new gtv.jq.Selection('skip');
        };

        global_mapping[project.keycodes.KEY_FF] = function(selectedItem, newSelected)
        {
            that.player.forward();
            return new gtv.jq.Selection('selected', jQuery('.pc_fw a'));
        };
        
        global_mapping[project.keycodes.KEY_RW] = function(selectedItem, newSelected)
        {
            that.player.rewind();
            return new gtv.jq.Selection('selected', jQuery('.pc_bw a'));
        };

        global_mapping[PAGE_UP] = global_mapping[project.keycodes.KEY_FF];
        global_mapping[PAGE_DOWN] = global_mapping[project.keycodes.KEY_RW];

        this.key_controller.setGlobalKeyMapping(global_mapping);

        this.key_controller.start(this.player_zone, true);

        jQuery('.pc_fullscreen a').click(function()
        {
            that.player.toggleFullscreenMode();
        });
        jQuery('.pc_fw a').click(function()
        {
            that.player.forward();
        });
        jQuery('.pc_bw a').click(function()
        {
            that.player.rewind();
        });
        jQuery('.pc_pause a').click(function()
        {
            that.player.pause();
            that.key_controller.setSelected(that.player.controls.play_btn_link);
        });
        jQuery('.pc_stop a').click(function()
        {
            that.player.stop();
        });
        jQuery('.pc_play a').click(function()
        {
            that.player.play();
            that.key_controller.setSelected(that.player.controls.pause_btn_link);
        });
    }
};

jQuery.extend(true, project.EpisodeDetails.prototype, new project.Log());

JsBehaviourToolkit.registerHandler('episode_details', project.EpisodeDetails);
