project.EpisodesOverview = function(dom_element, options)
{
    this.options = options || {};

    this.options.max_items = 4;

    this.dom_element = dom_element;

    this.show_title = this.dom_element.find('.js_headline');

    this.subnav = jQuery('#subnav');

    this.btn_latest_episodes = this.subnav.find('.js_btn_latest').first();
    this.btn_episodes = this.subnav.find('.js_btn_all').first();
    this.btn_back = this.subnav.find('.js_btn_back').first();

    this.latest_episodes_list_element = this.dom_element.find('ul.js_latest_episodes_list').first();
    this.episodes_list_element = this.dom_element.find('ul.js_all_episodes_list').first();

    this.logging_prefix = 'EpisodesOverview';

    this.is_list_latest_build = false;
    this.is_list_all_build = false;

    this.pager_element = jQuery('.pager').first();

    this.month_names = new Array("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");

    // Get params from URL-Hash
    this.show_id = jQuery.getUrlVar('showid');
    this.origin = jQuery.getUrlVar('origin') || 'shows';
    this.subpage = jQuery.getUrlHash('subpage') || 'latest_episodes';
    this.from = jQuery.getUrlHash('from') || 1;

    this.init();
    this.initializeNavigation();

    $(window).bind('hashchange', function()
    {
        window.location.reload();
    });
};

project.EpisodesOverview.prototype = {

    init: function()
    {
        var that = this;

        // Latest Episodes Button
        this.btn_latest_episodes.click(function(e)
            {
                e.preventDefault();
                that.showLatest();
                return false;
            }
        );

        // All Episodes Button
        this.btn_episodes.click(function(e)
            {
                e.preventDefault();
                that.showAll();
                return false;
            }
        );

        // Back Button
        this.btn_back.click(function(e)
            {
                e.preventDefault();
                project.navigateTo(that.origin + ".html");

                return false;
            }
        );

        if(this.subpage == 'latest_episodes')
        {
            this.showLatest(true);
        }
        else
        {
            this.showAll(true);
        }
    },

    /*
     * Gibt true zurück wenn der Hash neu gesetzt wurde, ansonsten false.
     */
    buildHash: function()
    {
        var from = (this.subpage == 'all_episodes') ? "&from=" + this.from : "";
        var new_hash = "subpage=" + this.subpage + from;

        if('#' + new_hash === window.location.hash)
        {
            return false;
        }
        else
        {
            window.location.hash = new_hash;
            return true;
        }
    },
    
    /*
     * Show latest episodes
     */
    showLatest: function(no_reload)
    {
        var no_reload = no_reload || false;
        this.subpage = "latest_episodes";
        
        /*
         * hier gehen wir nur rein wenn der Hash nicht verändert wurde.
         * Der Hash-Change wird nähmlich getriggert und erzwingt einen Reload der Seite
         */
        if(no_reload || !this.buildHash())
        { 
            this.list_element = this.latest_episodes_list_element;
            
            this.btn_episodes.parent().removeClass('active');
            this.btn_episodes.addClass('js_item');
            this.btn_latest_episodes.parent().addClass('active');
            this.btn_latest_episodes.removeClass('js_item');
            
            this.pager_element.hide();
            this.episodes_list_element.hide();
            
            this.retrieveJson(1);
        }
    },

    /*
     * show all episodes
     */
    showAll: function(no_reload)
    {
        var no_reload = no_reload || false;
        this.subpage = "all_episodes";

        /*
         * hier gehen wir nur rein wenn der Hash nicht verändert wurde.
         * Der Hash-Change wird nähmlich getriggert und erzwingt einen Reload der Seite
         */        
        if(no_reload || !this.buildHash())
        {
            this.list_element = this.episodes_list_element;
            
            this.btn_latest_episodes.parent().removeClass('active');
            this.btn_latest_episodes.addClass('js_item');
            this.btn_episodes.parent().addClass('active');
            this.btn_episodes.removeClass('js_item');
            
            this.latest_episodes_list_element.hide();
            
            this.retrieveJson(this.from);
            this.pager_element.show();
        }
    },

    /*
     * retrieve Json and Build List
     */
    retrieveJson: function(from, retry)
    {
        retry = retry || 0;
        var size_string = "&size=" + this.options.max_items;

        /* wenn fehler aufgetreten sind: einmal ohne size probieren */
        if (retry > 0)
        {
            retry = -1;
            size_string = '';
        }

        var that = this;
        project.data_provider.callWebApi(this.subpage + "&from=" + from + size_string + "&showid=" + this.show_id, function(data)
        {
            if (data.error)
            {
                console.error(data.error);
                that.retrieveJson(from, retry + 1);
                return;
            }

            if (!data.episodes || !(data.episodes.length > 0))
            {
                that.key_controller.start(that.nav_zone, true);
                return;
            }

            // build new List
            that.buildEpisodes(data, from);
        });
    },

    buildEpisodes: function(data, from)
    {
        var that = this;

        /*
         * Set Show-Title
         */
        if (data.episodes && data.episodes.length > 0)
        {
            that.show_title.text(data.episodes[0].show_title);

            // Webtrends Tracking
            if (this.subpage == "all_episodes")
            {
                var page = null;
                
                if (from > 1)
                {
                    page = 'Page ' + Math.ceil(from / this.options.max_items);
                }
                project.track('All Episodes', 'Shows', data.episodes[0].show_title, 'All Episodes', page);
            }
            else
            {
                project.track('Latest Episodes', 'Shows', data.episodes[0].show_title, 'Latest Episodes');
            }
        }

        this.pager = new project.Pager({
            'dom_element' : this.pager_element,
            'from' : this.from,
            'items_per_page' : this.options.max_items,
            'number_of_results' : data.number_of_results,
            'callback' : function(from)
            {
                that.from = from;
                /*
                 * workaround für samsung da hashchange Event nicht geworfen wird
                 */
                if (that.buildHash())
                {
                    window.location.reload();
                }
            }
        });

        this.key_controller.addBehaviorZone(this.pager_zone);

        /*
         * Episodes
         */
        var list_childs = this.list_element.children('li');
        // hide all List Elements
        list_childs.hide();
        list_childs.find('.js_item').removeClass('js_item');

        this.list_element.show();

        jQuery.each(data.episodes, function(idx, episode)
        {
            /*
             * Episode
             */
            var li_ele = jQuery(list_childs[idx]);

            li_ele.find('.js_sei_image').attr({
                'src': encodeURI(episode.image_url),
                'alt': episode.title
            });

            li_ele.find('.img_wrapper').attr('href', 'episode_detail.html?origin=episodes&episodeid=' + episode.id).addClass('js_item');
            li_ele.find('.js_sei_headline').html(project.string_toolkit.createCleanExcerpt(episode.title, 40));
            li_ele.find('.js_sei_subheadline').html(episode.episode_number + ': ' + episode.title);
            li_ele.find('.js_sei_description').html(project.string_toolkit.createCleanExcerpt(episode.text, 90));

            var on_air_date = episode.airdate || episode.air_date;
            on_air_date = Date.parse(on_air_date);
            on_air_date = new Date(on_air_date);
            on_air_date = that.month_names[on_air_date.getMonth()] + ' ' + on_air_date.getDate() + ', ' + on_air_date.getFullYear();
            li_ele.find('.js_sei_date').text(on_air_date);

            // show Element
            li_ele.show();

            if((idx + 1) === data.episodes.length)
            {
                if(that.subpage != "latest_episodes")
                {
                    // Select First Element on All Episodes Zone
                    that.key_controller.setSelected(that.episodes_list_element.find('.js_item').first());
                }
            }
        });
    },

    initializeNavigation: function()
    {
        var that = this;
        this.key_controller = new gtv.jq.KeyController();

        var nav_key_mapping = {};

        nav_key_mapping[project.keycodes.KEY_UP] =  function(selectedItem, newSelected)
        {
            newSelected = jQuery('.pager').find('.js_item').first();
            if (newSelected.length == 0)
            {
                newSelected = that.list_element.find('.js_item:visible').last();
            }

            if (newSelected.length == 0)
            {
                newSelected = jQuery('#subnav').find('.js_item').first();
            }
            return new gtv.jq.Selection('selected', newSelected);
        };

        var subnav_key_mapping = {};

        subnav_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            selectedItem.click();
            selectedItem.click(); //workaround for samsung
            return new gtv.jq.Selection('none');
        };
        subnav_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('skip');
        };
        subnav_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('skip');
        };
        subnav_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            var list_element = that.list_element.find('.js_item:visible').last();

            if (list_element.length != 0)
            {
                return new gtv.jq.Selection('selected', list_element);
            }
            return new gtv.jq.Selection('selected', selectedItem);
        };

        var episodes_key_mapping = {};

        episodes_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            if (selectedItem.is(that.list_element.find('.js_item').first()))
            {
                newSelected = that.list_element.find('.js_item').last();
            }
            return new gtv.jq.Selection('selected', newSelected);
        };
        episodes_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('#subnav').find('.js_item').first();
            return new gtv.jq.Selection('selected', newSelected);
        };
        episodes_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            if (selectedItem.is(that.list_element.find('.js_item').last()))
            {
                newSelected = that.list_element.find('.js_item').first();
            }
            return new gtv.jq.Selection('selected', newSelected);
        };
        episodes_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('.pager').find('.js_item').first();
            if (newSelected.length === 0)
            {
                newSelected = jQuery('#nav').find('.js_item').first();
            }

            return new gtv.jq.Selection('selected', newSelected);
        };

        var pager_key_mapping = {};

        pager_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            var list_element = that.list_element.find('.js_item:visible').last();

            if (list_element.length != 0)
            {
                return new gtv.jq.Selection('selected', list_element);
            }
            return new gtv.jq.Selection('none');
        };

        pager_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            selectedItem.click();
            return new gtv.jq.Selection('none');
        };

        this.nav_zone = project.getMenuBehaviourZone(nav_key_mapping);
        this.subnav_zone = project.getSubMenuBehaviourZone(subnav_key_mapping);
        this.pager_zone = project.getPagerBehaviourZone(pager_key_mapping);

        this.all_episodes_zone = project.getDefaultBehaviourZone('.js_all_episodes_list', episodes_key_mapping);
        this.latest_episodes_zone = project.getDefaultBehaviourZone('.js_latest_episodes_list', episodes_key_mapping);

        this.key_controller.addBehaviorZone(this.nav_zone);
        this.key_controller.addBehaviorZone(this.subnav_zone);
        this.key_controller.addBehaviorZone(this.pager_zone);

        this.key_controller.addBehaviorZone(this.latest_episodes_zone);
        this.key_controller.addBehaviorZone(this.all_episodes_zone);

        this.key_controller.start(this.latest_episodes_zone, true);
    }
};

jQuery.extend(true, project.EpisodesOverview.prototype, new project.Log());

JsBehaviourToolkit.registerHandler('episodes_overview', project.EpisodesOverview);
