project.Live = function(dom_element, options)
{
    var that = this;
    this.dom_element = dom_element;

    this.options = options || {};

    this.options.max_items = 4;

    this.min_epg_hours = 4;
    this.epg_hours = 4;
    this.max_epg_hours = 24;

    this.refresh_shows_after_fullscreen = false;

    this.logging_prefix = 'Live';

    this.trace(arguments);

    this.player = new project.Player({
        'url': project.livestream_url,
        'size': {
            'start_x': 176,
            'start_y': 86,
            'screen_width': 284,
            'screen_height': 524
        },
        'tracking_title': 'Livestream',
        'tracking_section': 'Live',
        'tracking_subsection1': 'Livestream'
    });

    this.playlist_wrapper = dom_element.find('.playlist_wrapper');
    this.coverflow_list = dom_element.find('.js_playlist');
    
    if (!this.coverflow_list)
    {
        return ;
    }

    this.shows = {
        'items': [],
        'index': 0
    };

    this.spots = {};

    for (var i = 1; i <= this.options.max_items; i++)
    {
        this.spots[i] = {
            'item': null,
            'img': this.coverflow_list.find('.pos' + i).find('.js_live_image'),
            'elem': this.coverflow_list.find('.pos' + i)
        };

        if (i === 1)
        {
            this.spots[i].elem.find('a').click(function()
            {
                that.rotateLeft();
            });
            this.spots[i].elem.click(function()
            {
                that.rotateLeft();
            });
        }

        if (i > 1)
        {
            this.spots[i].elem.click(function()
            {
                that.rotateRight();
            });
        }
    }
    
    this.label_now = dom_element.find('.js_now_showing');
    this.label_next = dom_element.find('.js_coming_next');

    this.label_next.find('.js_btn_back').click(function()
    {
        that.shows.index = 0;
        that.createView(that.shows.items.concat([]));
    });

    this.showShows();
    this.initializeNavigation();

    setTimeout(function()
    {
        that.player.play();
    }, 500);
};

project.Live.prototype = {

    showShows: function()
    {
        this.trace('showShows', arguments);

        var that = this;
        this.coverflow_list.show(function()
        {
            var self = that;

            project.data_provider.callWebApi('epg&hours=' + that.epg_hours, function(data)
            {
            	self.playlist_wrapper.hide();
                if (!data.clips || !(data.clips.length > 0))
                {
                    self.key_controller.start(self.nav_zone, true);
                    return;
                }
                self.key_controller.addBehaviorZone(self.shows_zone, false);

                self.trace('API call for shows completed: ', data);

                self.shows.items = data.clips;
                self.shows.index = 0;

                // deletes expired shows from json
                self.deleteExpiredClips();
            });
        });
    },

    /*
     * deletes expired shows from json
     */
    deleteExpiredClips: function()
    {
        this.trace('deleteExpiredClips', arguments);
        var that = this;

        project.data_provider.callWebApi('server_time', function(data)
        {
            if (data.server_time && data.server_time !== 'null')
            {
                for (var i = 0; i < that.shows.items.length; i++)
                {
                    var server_time = new Date(Date.parse(data.server_time));
                    var cur_shows_end_time = new Date(Date.parse(that.shows.items[0].endTime));

                    if (cur_shows_end_time < server_time)
                    {
                        project.trace('deleting expired clip');

                        that.shows.items.splice(0, 1);
                    }
                    else
                    {
                        project.trace('finish cleaning up clips data');

                        that.display();
                        // resetting epg hours
                        that.epg_hours = that.min_epg_hours;
                        return;
                    }
                }

                // All clips are expired
                that.epg_hours += 2;
                if(that.epg_hours > that.max_epg_hours)
                {
                    project.trace('All clips are expired and reached max epg hours, something is wrong here... we stop trying now');
                    return;
                }
                project.trace('All clips are expired, trying again with epg hours ' + that.epg_hours);

                // try again
                that.showShows();
            }
        });
    },

    display: function()
    {
        this.trace('display ', arguments);
        var that = this;
		this.playlist_wrapper.show();
        this.coverflow_list.show();

        if (this.shows && this.shows.items && this.shows.items.length == 0)
        {
            this.trace('displayShows', 'No shows provided. Will not update view.');
            return;
        }

        this.createView(this.shows.items.concat([]), function() { that.startShowTimer(); });
    },
    
    createView: function(shows, callback)
    {
    	var that = this;

		this.coverflow_list.show();
		
        if (shows && shows.length === 0)
        {
            return;
        }

        var i = this.shows.index;

        if (0 == i)
        {
            this.label_now.show();
            this.label_next.hide();
        }
        else
        {
            this.label_next.show();
           	this.label_now.hide();
        }

        for (var n = 1; n <= this.options.max_items; n++)
        {
            this.spots[n].item = shows[i] || null;
            i++;
        }

        //this.trace('spots in createView', this.spots);

        for (var n = 1; n <= this.options.max_items; n++)
        {
            if (this.spots[n].item == null)
            {
                this.spots[n].elem.hide();
            }
            else
            {
                this.spots[n].elem.show();
                this.spots[n].img.attr('src', this.spots[n].item.image);

                if (n == 1)
                {
                    if (this.spots[n].item.type == 'live')
                    {
                        jQuery('body').addClass('live_event');
                    }
                    else
                    {
                        jQuery('body').removeClass('live_event');
                    }

                    this.spots[n].elem.find('.js_live_title').html(this.spots[n].item.title);

                    var description = project.string_toolkit.createCleanExcerpt(this.spots[n].item.archiveText, 100);
                    this.spots[n].elem.find('.js_live_desc').html(description);
                }
            }
        }

        if(typeof callback != "undefined")
        {
            callback();
        }
    },

    startShowTimer: function()
    {
        this.trace('startShowTimer');
        var that = this;

        // dont make api calls in fullscreen mode
        if(this.player.fullscreen)
        {
            project.trace('new livestream, updating when leaving fullscreen-mode');
            this.refresh_shows_after_fullscreen = true;
            return;
        }

        if (this.shows.items.length > 4)
        {
            // we have enough shows to display in upcoming screen, but we should really
            // try to switch to the next live show when it is about to start. That's why
            // we set us a notification event in X milliseconds in the future from NOW
            // (server time) to the next shows' start time
            project.data_provider.callWebApi('server_time', function(data)
            {
                if (data.server_time && data.server_time !== 'null')
                {
                    var server_time = new Date(Date.parse(data.server_time));
                    var next_shows_start_time = new Date(Date.parse(that.shows.items[1].startTime));
                    var diff = next_shows_start_time.getTime() - server_time.getTime();
                    
                    project.trace('until next show: ' + diff + ' time: ' + that.shows.items[1].startTime);
                    
                    setTimeout(function()
                    {
                        project.trace('re-creating view as new show should be visible now');
                        
                        that.shows.items.splice(0, 1);
                        if (that.shows.index > 0)
                        {
                            that.shows.index -= 1; // stay on current position if not on first position anyways
                        }
                        that.createView(that.shows.items, function() { that.startShowTimer(); });
                    }, diff);
                }
            });
        }
        else
        {
            project.trace('too less shows left from EPG...trying to refresh our data (items.length: ' + this.shows.items.length + ')');
            this.showShows();
        }
    },

    rotateRight: function()
    {
        this.trace('rotateRight');

        if (this.shows.index < this.shows.items.length - 1)
        {
            this.shows.index += 1;
            this.createView(this.shows.items.concat([]));
        }
    },

    rotateLeft: function()
    {
        this.trace('rotateLeft');

        if (this.shows.index > 0)
        {
            this.shows.index += -1;
            this.createView(this.shows.items.concat([]));
        }
    },

    initializeNavigation: function()
    {
        var that = this;

        this.key_controller = new gtv.jq.KeyController();

        var nav_key_mapping = {};

        nav_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            var skip_button = jQuery('.skip_buttons').find('.js_item').first();
            if (skip_button.is(':visible'))
            {
                return new gtv.jq.Selection('selected', skip_button);
            }

            var play_btn = jQuery('.pc_fullscreen a');
            return new gtv.jq.Selection('selected', play_btn);
        };

        nav_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('selected', jQuery('.pc_fullscreen a'));
        };

        var coverflow_head_key_mapping = {};
        coverflow_head_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {  
            selectedItem.click();
            var playlist_item = jQuery('.skip_buttons').find('.js_item').first();
            if (playlist_item.length) 
            {
                return new gtv.jq.Selection('selected', playlist_item);
            }

            return new gtv.jq.Selection('none');
        };

        coverflow_head_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('skip');
        };

        coverflow_head_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            var play_btn = jQuery('.pc_fullscreen a');
            return new gtv.jq.Selection('selected', play_btn);
        };

        coverflow_head_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('skip');
        };

        coverflow_head_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            var playlist_item = jQuery('.skip_buttons').find('.js_item').first();
            return new gtv.jq.Selection('selected', playlist_item);
        };

        var player_key_mapping = {};
        player_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            var parent = selectedItem.parent();

            if (that.player && parent.hasClass('pc_fullscreen'))
            {
                that.player.toggleFullscreenMode();
                selectedItem = jQuery('#player_controls').find('.js_item').first();
            }
            else if (that.player && parent.hasClass('pc_pause'))
            {
                if (that.player.pause())
                {
                    selectedItem = jQuery('.pc_play a');
                }
            }
            else if (that.player && parent.hasClass('pc_stop'))
            {
                that.player.stop();
            }
            else if (that.player && parent.hasClass('pc_play'))
            {
                that.player.play();
                selectedItem = jQuery('.pc_pause a');
            }

            return new gtv.jq.Selection('selected', selectedItem);
        };

        player_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            if (selectedItem.is(jQuery('#player_controls .js_item').first()))
            {
                return new gtv.jq.Selection('selected', jQuery('#player_controls .js_item').last());
            }
            return new gtv.jq.Selection('none');
        };

        player_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            if (that.player.fullscreen)
            {
                return new gtv.jq.Selection('skip');
            }

            return new gtv.jq.Selection('selected', jQuery('#nav .js_item').last());
        };

        player_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            if (selectedItem.is(jQuery('#player_controls .js_item').last()))
            {
                return new gtv.jq.Selection('selected', jQuery('#player_controls .js_item').first());
            }
            return new gtv.jq.Selection('none');
        };

        player_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            if (that.player.fullscreen)
            {
                return new gtv.jq.Selection('skip');
            }

            var skip_right = jQuery('.js_skip_right');
            if (skip_right.is(':visible'))
            {
                return new gtv.jq.Selection('selected', skip_right);
            }

            var play_btn = jQuery('.pc_fullscreen a');
            return new gtv.jq.Selection('selected', play_btn);
        };

        var shows_key_mapping = {};
        shows_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            if(selectedItem.is('.js_skip_left'))
            {
                that.rotateLeft();
            }
            else if(selectedItem.is('.js_skip_right'))
            {
                that.rotateRight();
            }

            return new gtv.jq.Selection('selected', selectedItem);
        };

        shows_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            if(selectedItem.is('.js_skip_left')) 
            {
                return new gtv.jq.Selection('selected', jQuery('.js_skip_right'));
            }
            return new gtv.jq.Selection('none');
        };

        shows_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            var back_button = jQuery('.js_btn_back');
            if (back_button.is(':visible'))
            {
                return new gtv.jq.Selection('selected', back_button);
            }

            var play_btn = jQuery('.pc_fullscreen a');
            return new gtv.jq.Selection('selected', play_btn);
        };

        shows_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            if(selectedItem.is('.js_skip_right'))
            {
                return new gtv.jq.Selection('selected', jQuery('.js_skip_left'));
            }
            return new gtv.jq.Selection('none');
        };

        shows_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            var nav_item = jQuery('#nav').find('.js_item').first();
            return new gtv.jq.Selection('selected', nav_item);
        };

        this.nav_zone = project.getMenuBehaviourZone(nav_key_mapping);
        this.coverflow_head_zone = project.getDefaultBehaviourZone('.js_coming_next', coverflow_head_key_mapping);
        this.shows_zone = project.getDefaultBehaviourZone('.skip_buttons', shows_key_mapping);
        this.player_zone = project.getDefaultBehaviourZone('#player_controls', player_key_mapping);

        this.key_controller.addBehaviorZone(this.nav_zone);
        this.key_controller.addBehaviorZone(this.player_zone);
        this.key_controller.addBehaviorZone(this.coverflow_head_zone);
        this.key_controller.addBehaviorZone(this.shows_zone);

        var global_mapping = {};
        global_mapping[project.keycodes.KEY_PLAY] = function(selectedItem, newSelected)
        {
            that.player.play();
            return new gtv.jq.Selection('selected', jQuery('.pc_pause a'));
        };

        global_mapping[project.keycodes.KEY_STOP] = function(selectedItem, newSelected)
        {
            if (that.player.stop())
            {
                return new gtv.jq.Selection('selected', jQuery('.pc_stop a'));
            }
            return new gtv.jq.Selection('skip');                
        };

        global_mapping[project.keycodes.KEY_PAUSE] = function(selectedItem, newSelected)
        {
            if (that.player.pause())
            {
                return new gtv.jq.Selection('selected', jQuery('.pc_play a'));
            }
            return new gtv.jq.Selection('skip');    
        };

        this.key_controller.setGlobalKeyMapping(global_mapping);

        this.key_controller.start(this.player_zone, true);

        jQuery('.js_skip_left').click(function()
        {
            that.rotateLeft();
        });
        jQuery('.js_skip_right').click(function()
        {
            that.rotateRight();
        });

        jQuery('.pc_fullscreen a').click(function()
        {
            that.player.toggleFullscreenMode();
        });
        jQuery('.pc_pause a').click(function()
        {
            if (that.player.pause())
            {
                that.key_controller.setSelected(that.player.controls.play_btn_link);
            }
        });
        jQuery('.pc_stop a').click(function()
        {
            that.player.stop();
        });
        jQuery('.pc_play a').click(function()
        {
            that.player.play();
            that.key_controller.setSelected(that.player.controls.pause_btn_link);
        });
    }
};

jQuery.extend(true, project.Live.prototype, new project.Log());

JsBehaviourToolkit.registerHandler('live', project.Live);
