project.LiveTeaser = function(dom_element, options)
{
    this.dom_element = dom_element;
    this.live_teaser_ele = this.dom_element.find('.js_item_row');
    
    this.logging_prefix = 'LiveTeaser';
    this.clips = null;
    
    this.min_hours = 4;
    this.epg_hours = 4;
    this.max_epg_hours = 24;

    this.epg_retrieves = 0;
    this.epg_max_retrieves = 3;
    
    this.server_time_retrieves = 0;
    this.server_time_max_retrieves = 3;
    
    this.retrieveData();
};

project.LiveTeaser.prototype = {

        retrieveData: function()
        {
            this.trace('retrieveData', arguments);
            // resetting server_time retrieves
            this.server_time_retrieves = 0;

            if(this.epg_retrieves >= this.epg_max_retrieves)
            {
                project.trace('too many retrieves, stopping now');
                this.dom_element.removeClass('preloading');
                this.dom_element.text('Connection timed out! Please check your internet connection or try again later.');
                return;
            }
            else
            {
                this.epg_retrieves++;
            }

            var that = this;
            
            project.data_provider.callWebApi("epg&hours=" + this.epg_hours, function(data)
            {
                if (!data.clips || !(data.clips.length > 0))
                {
                    return;
                }
                
                that.clips = data.clips;
                that.deleteExpiredClips();       
            },
            function(jqXHR, textStatus, errorThrown) // error callback
            {
                that.retrieveData();
            });
        },
        
        deleteExpiredClips: function()
        {
            this.trace('deleteExpiredClips', arguments);
            // resetting epg retrieves
            this.epg_retrieves = 0;
            if(this.server_time_retrieves >= this.server_time_max_retrieves)
            {
                project.trace('too many retrieves, stopping now');
                return;
            }
            else
            {
                this.server_time_retrieves++;
            }
            var that = this;
            
            project.data_provider.callWebApi('server_time', function(data)
            {
                if (data.server_time && data.server_time !== 'null')
                {
                    for (var i = 0; i < that.clips.length; i++)
                    {
                        var server_time = new Date(Date.parse(data.server_time));
                        var cur_shows_end_time = new Date(Date.parse(that.clips[0].endTime));
                        
                        if (cur_shows_end_time < server_time)
                        {
                            project.trace('removing expired clip');
                            
                            that.clips.splice(0, 1);
                        }
                        else
                        {
                            project.trace('finish cleaning up clips data');
                            
                            that.buildLiveTeaser();
                            // resetting epg hours
                            that.epg_hours = that.min_epg_hours;
                            return;
                        }
                    }
                    
                    // All clips are expired
                    that.epg_hours += 2;
                    if(that.epg_hours > that.max_epg_hours)
                    {
                        project.trace('All clips are expired and reached max epg hours, something is wrong here... we stop trying now');
                        return;
                    }
                    project.trace('All clips are expired, trying again with epg hours ' + that.epg_hours);
                    
                    // try again
                    that.retrieveData();                            
                }
            });
        },
        
        buildLiveTeaser: function()
        {
            this.trace('buildLiveTeaser', arguments);
            var that = this;
            var clip = this.clips[0];
            
            this.live_teaser_ele.find('.js_lt_image').attr({
                'src': clip.image,
                'alt': clip.title
            });
            this.live_teaser_ele.find('.js_lt_headline').text(clip.title);
            this.live_teaser_ele.find('.js_lt_description').html(project.string_toolkit.createCleanExcerpt(clip.archiveText, 100));
            
            this.dom_element.removeClass('preloading');
            this.live_teaser_ele.removeClass('hide');
            
            if (this.clips.length > 0)
            {
                // we have enough shows to display in upcoming screen, but we should really
                // try to switch to the next live show when it is about to start. That's why
                // we set us a notification event in X milliseconds in the future from NOW
                // (server time) to the next shows' start time
                project.data_provider.callWebApi('server_time', function(data)
                {
                    if (data.server_time && data.server_time !== 'null')
                    {
                        
                        var server_time = new Date(Date.parse(data.server_time));
                        var next_shows_start_time = new Date(Date.parse(that.clips[1].startTime));
                        var diff = next_shows_start_time.getTime() - server_time.getTime();
                        
                        project.trace('until next show: '+diff+' time: '+that.clips[1].startTime);
                        
                        setTimeout(function()
                        {
                            project.trace('re-creating view as new show should be visible now');
                            that.clips.splice(0, 1);
                            
                            that.buildLiveTeaser();
                        }, diff);
                    }
                });
            }
            else
            {
                project.trace('too less shows left from EPG...trying to refresh our data');
                that.retrieveData();
            }            
        }
};

jQuery.extend(true, project.LiveTeaser.prototype, new project.Log());

JsBehaviourToolkit.registerHandler('live_teaser', project.LiveTeaser);
