project.Pager = function(options)//dom_element, items_per_page, number_of_results, callback)
{
    this.dom_element = options.dom_element;
    this.items_per_page = options.items_per_page;
    var page = (options.from > 1) ? ((options.from - 1) / this.items_per_page) : 0;
    this.cur_page = (page + 1);
    this.pages = 0;
    this.number_of_results = options.number_of_results;
    this.callback = options.callback;
    
    this.init();
};

project.Pager.prototype = {
    
    init: function()
    {
        this.pages = Math.ceil(this.number_of_results / this.items_per_page);
        this.builtPagination();
    },

    prev: function()
    {
        if (this.cur_page > 1)
        {
            this.showPage(this.cur_page - 1);
        }
    },
    
    next: function()
    {
        if (this.cur_page < this.pages)
        {
            this.showPage(this.cur_page + 1);
        }
    },
    
    showPage: function(page_id)
    {
        var from = (page_id - 1) * this.items_per_page + 1;
        //var to = from + this.items_per_page - 1;
        
        this.cur_page = page_id;
        this.builtPagination();
        this.callback(from);
    },
    
    builtPagination: function()
    {
        var that = this;
        var i = 1;
        var js_item;
        this.dom_element.empty();
        if(this.pages <= 1)
        {
            this.dom_element.hide();
            return;
        }
        var ellip = '<li class="ellip">&hellip;</li>';
        
        /*
         * skip to previous
         */
        if(this.cur_page > 1)
        {
            js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>skip left</span></a>').click(function(event)
            {
                event.preventDefault();
                that.prev();
            });             
            this.dom_element.append(jQuery('<li class="skip_left js_item_parent"></li>').append(js_item));
        }
        
        /*
         * Pages 1-3 + Last
         */
        if(this.pages < 4)
        {
            i = 1;
            
            do
            {
                if(this.cur_page === i)
                {
                    // active
                    this.dom_element.append(jQuery('<li class="active"><strong><span>' + i + '</span></strong></li>'));
                }
                else
                {
                    js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>' + i + '</span></a>').click({ page_nr: i }, function(event)
                    {
                        event.preventDefault();
                        that.showPage(event.data.page_nr);
                    });                    
                    this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
                }
                
                i++;
                
            } while(i < 4 && i < (this.pages + 1));
        }
        /*
         * Pages 1-3 + Last
         */
        else if(this.cur_page < 4)
        {
            i = 1;
            
            do
            {
                if(this.cur_page === i)
                {
                    // active
                    this.dom_element.append(jQuery('<li class="active"><strong><span>' + i + '</span></strong></li>'));
                }
                else
                {
                    js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>' + i + '</span></a>').click({ page_nr: i }, function(event)
                    {
                        event.preventDefault();
                        that.showPage(event.data.page_nr);
                    });
                    this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
                }
                
                i++;

            } while(i < 4 && i < this.pages);
            
            /*
             * Seperator "..."
             */
            this.dom_element.append(jQuery(ellip));  
            
            /*
             * last Page
             */
            js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>' + this.pages + '</span></a>').click(function(event)
            {
                event.preventDefault();
                that.showPage(that.pages);
            });
            this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));           
        }
        /*
         * Pages 4-X
         */        
        else if(this.cur_page > 3 && this.pages > 3 && (this.cur_page + 2) < this.pages)
        {
            /*
             * first Page
             */
            js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>1</span></a>').click(function(event)
            {
                event.preventDefault();
                that.showPage(1);
            });
            this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
            
            /*
             * Seperator "..."
             */
            this.dom_element.append(jQuery(ellip));
            
            i = this.cur_page;
            
            do
            {
                if(this.cur_page === i)
                {
                    // active
                    this.dom_element.append(jQuery('<li class="active"><strong><span>' + i + '</span></strong></li>'));
                }
                else
                {
                    js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>' + i + '</span></a>').click({ page_nr: i }, function(event)
                    {
                        event.preventDefault();
                        that.showPage(event.data.page_nr);
                    });
                    this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
                }
                
                i++;

            } while(i < (this.cur_page + 3) && i < this.pages);

            /*
             * Seperator "..."
             */
            if((this.cur_page + 3) < this.pages)
            {
                this.dom_element.append(jQuery(ellip));      
            }
            
            /*
             * last Page
             */
            js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>' + this.pages + '</span></a>').click(function(event)
            {
                event.preventDefault();
                that.showPage(that.pages);
            });            
            this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
        }
        /*
         * Pages X-Last
         */        
        else if((this.cur_page + 3) >= this.pages)
        {
            /*
             * first Page
             */
            js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>1</span></a>').click(function(event)
            {
                event.preventDefault();
                that.showPage(1);
            });
            this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
            
            /*
             * Seperator "..."
             */
            this.dom_element.append(jQuery(ellip));
            
            for(i = (this.pages - 2); i < (this.pages + 1); i++)
            {
                if(this.cur_page === i)
                {
                    // active
                    this.dom_element.append(jQuery('<li class="active"><strong><span>' + i + '</span></strong></li>'));
                }
                else
                {
                    js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>' + i + '</span></a>').click({ page_nr: i }, function(event)
                    {
                        event.preventDefault();
                        that.showPage(event.data.page_nr);
                    });
                    this.dom_element.append(jQuery('<li class="js_item_parent"></li>').append(js_item));
                }
            }
        }
        
        /*
         * skip to next
         */
        if(this.cur_page < this.pages)
        {
            js_item = jQuery('<a class="js_item" href="javascript: void(0);"><span>skip right</span></a>').click(function(event)
            {
                event.preventDefault();
                that.next();
            });
            this.dom_element.append(jQuery('<li class="skip_right js_item_parent"></li>').append(js_item));
        }
        
        this.dom_element.show();
    }
};