project.PlayerControls = function(wrapper_id)
{
    this.wrapper_id = wrapper_id || 'player_controls';
    this.wrapper = jQuery('#' + this.wrapper_id);

    this.fullscreen_hide_delay = 5000;

    this.btn_list = this.wrapper.find('ul');

    this.play_btn = this.wrapper.find('.pc_play');
    this.pause_btn = this.wrapper.find('.pc_pause');
    this.stop_btn = this.wrapper.find('.pc_stop');
    this.fullscreen_btn = this.wrapper.find('.pc_fullscreen');

    this.visible = true;
    this.timer = 0;
};

project.PlayerControls.prototype = {

    hide: function()
    {
        var that = this;

        this.visible = false;
        this.wrapper.hide();

        jQuery("body").bind("mousemove", function(event)
        {
            if ( false === that.visible )
            {
                that.show();
                jQuery("body").unbind("mousemove");
            }
        });
    },

    show: function()
    {
        this.visible = true;
        this.wrapper.show();
    },

    startHideTimer: function()
    {
        var that = this;

        this.timer = setTimeout(function()
        {
            that.hide();
        }, that.fullscreen_hide_delay);
    },

    stopHideTimer: function()
    {
        clearTimeout(this.timer);
    }
};

project.PlayerVolumeDisplay = function(start_volume)
{
    this.hide_delay = 1500;
    this.volume_filler_css_offset = 73;

    this.volume_bar = jQuery('#volume_bar');
    this.volume_filler = this.volume_bar.find('div');

    this.volume_filler.css('width', this.volume_filler_css_offset + start_volume);

    this.visible = true;
    this.timer = 0;
};

project.PlayerVolumeDisplay.prototype = {

    hide: function()
    {
        this.visible = false;
        this.volume_bar.hide();
    },

    show: function()
    {
        this.visible = true;
        this.volume_bar.show();
    },

    startHideTimer: function()
    {
        var that = this;

        this.timer = setTimeout(function()
        {
            that.hide();
        }, that.hide_delay);
    },

    stopHideTimer: function()
    {
        clearTimeout(this.timer);
    },

    setVolumeFilling: function(volume)
    {
        this.stopHideTimer();

        this.volume_filler.css('width', this.volume_filler_css_offset + Math.floor(volume / 10) * 10);

        this.show();
        this.startHideTimer();
    }
};

project.Player = function(options)
{
    var that = this;
    this.logging_prefix = 'Player';

    this.controls = new project.PlayerControls();
    this.options = options || {};

    this.tracking_title = options.tracking_title || null;
    this.tracking_section = options.tracking_section || '';
    this.tracking_subsection1 = options.tracking_subsection1 || null;
    this.tracking_subsection2 = options.tracking_subsection2 || null;

    this.player_id = options.player_id || 'video';
    this.audio_id = options.player_id || 'audio';
    this.video = document.getElementById(this.player_id);
    this.audio = document.getElementById(this.audio_id);

    this.volume_display = new project.PlayerVolumeDisplay(this.audio.GetVolume());
    this.error_display = jQuery('.js_error');
    
    this.plugin = new Common.API.Plugin();
	this.state = 0;
    this.fullscreen = false;
    this.mute = this.audio.GetUserMute();

    this.setKeyHandler();

    /*
     * workaround for samsung maple browser 
     * global callbacks, poor but necessary
     */
    project.player = this;
    this.video.OnRenderingComplete = 'project.player.onRenderingComplete';
    this.video.OnRenderError = 'project.player.onError';
    this.video.OnNetworkDisconnected = 'project.player.onNetworkDisconnected';
    this.video.OnConnectionFailed = 'project.player.onConnectionFailed';
};

project.Player.prototype = {

    onRenderingComplete: function()
    {
        this.stop();
    },

    onNetworkDisconnected: function()
    {
        this.setError('Network Disconnected.');
    },

    onConnectionFailed: function()
    {
        this.setError('Connection failed.');
    },

    onError: function(code)
    {
        switch (code)
        {
            case 1:
                this.setError('Unsupported container.');
                break;
            case 2:
                this.setError('Unsupported video codec.');
                break;
            case 3:
                this.setError('Unsupported audio codec.');
                break;
            case 4:
                this.setError('Unsupported video resolution.');
                break;
            default:
                this.setError('Unknown error.');
                break;
        }
    },

    setError: function(text)
    {
        this.trace('error: ' + text);
        this.error_display.text('Error: ' + text).removeClass('hide');
        this.stop();
    },

    toggleFullscreenMode: function()
    {
        if (this.fullscreen === true)
        {
            this.leaveFullscreenMode();
        }
        else
        {
            if (this.isPlaying() || this.isPaused())
            {
	            this.enterFullscreenMode();
	        }
       	}
    },

    enterFullscreenMode: function()
    {
        this.trace('enterFullscreenMode');
        this.track('fullscreen entered');

        jQuery("#player").hide();
        jQuery("#main").hide();
        jQuery("#header").hide();
        
        jQuery('body').addClass('fullscreen');
        jQuery('#page').addClass("no_bg");

		this.video.SetDisplayArea(0, 0, 960, 540);

        this.fullscreen = true;
        this.controls.startHideTimer();

        if (this.isPaused())
        {
            this.video.Pause();
        }
    },

    leaveFullscreenMode: function()
    {
        this.controls.stopHideTimer();
        this.controls.show();
        this.fullscreen = false;

        this.trace('leaveFullscreenMode');
    	this.track('fullscreen left');

    	jQuery("#player").show();
    	jQuery("#main").show();
        jQuery("#header").show();

        jQuery('body').removeClass('fullscreen');
        jQuery('#page').removeClass("no_bg");

        var size = this.options.size;
        this.video.SetDisplayArea(size.start_x, size.start_y, size.screen_height, size.screen_width);

        if (this.isPaused())
        {
            this.video.Pause();
        }
    },

    play: function()
    {
       if(this.isStopped())
       {
       		if (this.fullscreen)
       		{
       		    this.video.SetDisplayArea(0, 0, 960, 540);
       		}
       		else
       		{
       		    var size = this.options.size;
       		    this.video.SetDisplayArea(size.start_x, size.start_y, size.screen_height, size.screen_width);
       		}

       		this.error_display.addClass('hide');
       		this.video.Play(this.options.url);

       		jQuery('#player').removeClass("player_background");
       }
       else if(this.isPaused())
       {
       		this.video.Resume();
       }

       if (this.isStopped() || this.isPaused())
       {
           this.state = 1;
           this.trace('play:', this.options.url);
           this.track('playing');
           this.controls.play_btn.hide();
           this.controls.pause_btn.show();
           this.plugin.setOnScreenSaver(99999);
           this.plugin.setOffScreenSaver();
       }
    },

    pause: function()
    {
    	if(this.isPlaying())
    	{
            var success = this.video.Pause();

            this.trace(success);

            if (!success)
            {
                return false;
            }  

    		 this.trace('pause');
    		 this.track('paused');
    		 
    		 this.controls.play_btn.show();
    		 this.controls.pause_btn.hide();
    		 this.state = 2;
    	}

    	return success || false;
    },

    stop: function()
    {
        if(this.isPlaying() || this.isPaused())
    	{
            var success = this.video.Stop();
            if (!success)
            {
                return false;
            }

    		this.trace('stop');
    		this.track('stopped');

        	jQuery('#player').addClass("player_background");
        	this.controls.play_btn.show();
        	this.controls.pause_btn.hide();

        	this.state = 0;

            if (this.fullscreen)
            {
                this.leaveFullscreenMode();
            }
       }

        return success || false;
    },

    rewind: function()
    {
        this.video.JumpBackward(5);
    },

    forward: function()
    {
        this.video.JumpForward(5);
    },

    isPaused: function()
    {
        return (this.state === 2);
    },

    isPlaying: function()
    {
        return (this.state === 1);
    },

    isStopped: function()
    {
        return (this.state === 0);
    },

    hideControls: function()
    {
        this.controls.hide();
    },

    showControls: function()
    {
        this.controls.show();
    },

    track: function(action)
    {
        if (this.tracking_title)
        {
            if (this.tracking_subsection2)
            {
                project.track(this.tracking_title, this.tracking_section, this.tracking_subsection1, this.tracking_subsection2, action);
            }
            else
            {
                project.track(this.tracking_title, this.tracking_section, this.tracking_subsection1, action);
            }
        }
    },

    setKeyHandler: function()
    {
        var that = this;
        var F8 = 119;
        var F9 = 120;
        var F10 = 121;

        jQuery('body').keyup(function(e)
        {
            if (e.keyCode == project.keycodes.KEY_MUTE || e.keyCode == F8)
            {
                if (that.mute)
                {
                    that.volume_display.setVolumeFilling(that.audio.GetVolume());
                    that.trace('unmute');
                    that.mute = false;
                    that.audio.SetUserMute(0);
                }
                else
                {
                    that.mute = true;
                    that.volume_display.setVolumeFilling(0);
                    that.trace('mute');
                    that.audio.SetUserMute(1);
                }
            }
            else if (e.keyCode == project.keycodes.KEY_PANEL_VOL_UP || e.keyCode == project.keycodes.KEY_VOL_UP || e.keyCode == F10)
            {
                that.audio.SetVolume(that.audio.GetVolume() + 1);
                that.volume_display.setVolumeFilling(that.audio.GetVolume());
                that.trace('lauter', that.audio.GetVolume());
                that.mute = false;
                that.audio.SetUserMute(0);
            }
            else if (e.keyCode == project.keycodes.KEY_PANEL_VOL_DOWN || e.keyCode == project.keycodes.KEY_VOL_DOWN || e.keyCode == F9)
            {
                that.audio.SetVolume(that.audio.GetVolume() - 1);
                that.volume_display.setVolumeFilling(that.audio.GetVolume());
                that.trace('leiser', that.audio.GetVolume());
                that.mute = false;
                that.audio.SetUserMute(0);
            }
            else if (e.keyCode == project.keycodes.KEY_RETURN || e.keyCode == project.keycodes.KEY_PANEL_RETURN)
            {
                that.leaveFullscreenMode();
            }
            else if (that.fullscreen)
            {
                var is_navigation_key = (e.keyCode == project.keycodes.KEY_ENTER) || (e.keyCode == project.keycodes.KEY_RIGHT) || 
                                        (e.keyCode == project.keycodes.KEY_LEFT) || (e.keyCode == project.keycodes.KEY_UP) ||
                                        (e.keyCode == project.keycodes.KEY_DOWN);

                if (!that.controls.visible && is_navigation_key)
                {
                    e.stopPropagation();
                }

                if (that.controls.visible === false)
                {
                    that.controls.show();
                    e.preventDefault();
                }

                that.controls.stopHideTimer();
                that.controls.startHideTimer();
            }            
        });
    }
};

jQuery.extend(true, project.Player.prototype, new project.Log());