
project.ShowsOverview = function(dom_element, options)
{
    this.logging_prefix = 'ShowsOverview';
    this.trace(arguments);

    var that = this;

    this.dom_element = dom_element;

    this.options = options || {};

    this.options.max_items = 7;
    
    this.subpage = jQuery.getUrlHash('subpage') || 'latest';

    this.show_title = this.dom_element.find('.js_headline');

    this.coverflow_list = jQuery(dom_element).find('#coverflow').find('ul');

    this.coverflow_list.hide();

    this.shows = {
        'latest': {
            'items': [],
            'index': 0
        },
        'popular': {
            'items': [],
            'index': 0
        }
    };

    var mid_index = Math.ceil(this.options.max_items / 2);

    this.spots = {};

    var elem;
    
    for (var i = 1; i <= this.options.max_items; i++)
    {
        elem = this.coverflow_list.find('.pos'+i);

        this.spots[i] = {
            'item': null,
            'img': elem.find('.js_scfi_image'),
            'elem': elem
        };

        if (i < mid_index)
        {
            this.spots[i].elem.click(function()
            {
                that.rotateLeft();
            });
        }
        else if (i > mid_index)
        {
            this.spots[i].elem.click(function()
            {
                that.rotateRight();
            });
        }
    };

    this.trace('spots', this.spots);

    this.btn_latest_shows = dom_element.find('.js_btn_latest');
    this.btn_popular_shows = dom_element.find('.js_btn_popular');

    this.btn_latest_shows.bind('click', function(e)
    {
        e.preventDefault();
        that.showLatest();
    });

    this.btn_popular_shows.bind('click', function(e)
    {
        e.preventDefault();
        that.showPopular();
    });

    jQuery('.js_skip_left').bind('click', function(e)
    {
        e.preventDefault();
        that.rotateLeft();
    });

    jQuery('.js_skip_right').bind('click', function(e)
    {
        e.preventDefault();
        that.rotateRight();
    });

    if (this.subpage == 'latest')
    {
        this.showLatest(true);
    }
    else
    {
        this.showPopular(true);
    }

    jQuery(window).bind('hashchange', function()
    {
        window.location.reload();
    });

    this.initializeNavigation();
};

project.ShowsOverview.prototype = {

    showShows: function()
    {
        this.trace('showShows', arguments);

        var that = this;

        if (this.shows[this.subpage].items && this.shows[this.subpage].items.length > 0)
        {
            this.trace('Not calling API for latest shows again.');
            this.display(this.shows[this.subpage]);
        }
        else
        {
            var command = ('latest' == that.subpage) ? 'all_shows' : 'popular_shows';
            this.show_title.show();

            project.data_provider.callWebApi(command, function(data)
            {
                that.key_controller.setZone(that.shows_zone, true);

                that.trace('API call for shows completed: ', data);

                that.shows[that.subpage].items = data.shows;
                that.shows[that.subpage].index = 0;

                that.display(that.shows[that.subpage]);
                that.coverflow_list.show();
            });
        }
    },

    /*
     * Gibt true zurück wenn der Hash neu gesetzt wurde, ansonsten false.
     */
    buildHash: function()
    {
        var new_hash = "subpage=" + this.subpage;
        
        if('#' + new_hash === window.location.hash)
        {
            return false;
        }
        else
        {
            window.location.hash = new_hash;
            return true;
        }
    },    
    
    showLatest: function(no_reload)
    {
        var no_reload = no_reload || false;
        this.subpage = 'latest';
        
        /*
         * hier gehen wir nur rein wenn der Hash nicht verändert wurde.
         * Der Hash-Change wird nähmlich getriggert und erzwingt einen Reload der Seite
         */        
        if(no_reload || !this.buildHash())
        {     
            this.btn_latest_shows.parent().addClass('active');
            this.btn_latest_shows.removeClass('js_item');
            this.btn_popular_shows.parent().removeClass('active');
            this.btn_popular_shows.addClass('js_item');

            this.showShows();
            project.track('Latest Shows', 'Shows', 'Latest Shows');
        }
    },

    showPopular: function(no_reload)
    {
        var no_reload = no_reload || false;        
        this.subpage = 'popular';
        
        /*
         * hier gehen wir nur rein wenn der Hash nicht verändert wurde.
         * Der Hash-Change wird nähmlich getriggert und erzwingt einen Reload der Seite
         */
        if(no_reload || !this.buildHash())
        {           
            this.btn_latest_shows.parent().removeClass('active');
            this.btn_latest_shows.addClass('js_item');
            this.btn_popular_shows.parent().addClass('active');
            this.btn_popular_shows.removeClass('js_item');
    
            this.showShows();
            project.track('Most Popular Shows', 'Shows', 'Most Popular Shows');
        }
    },    
    
    display: function(all_shows)
    {
        this.trace('display ', arguments);

        this.show_title.hide();
        this.coverflow_list.show();

        if (all_shows && all_shows.items && all_shows.items.length == 0)
        {
            this.trace('displayShows', 'No shows provided. Will not update view.');
            return;
        }

        this.createView(all_shows.items.concat([]));
    },

    createView: function(shows)
    {
        var that = this;

        //this.trace('createView: Will display the following '+shows.length+' shows: ', shows);

        if (shows && shows.length === 0)
        {
            return;
        }

        var mid_index = Math.ceil(this.options.max_items / 2);

        var i = that.shows[that.subpage].index - Math.floor(this.options.max_items / 2);

        for (var n = 1; n <= this.options.max_items; n++)
        {
            this.spots[n].item = shows[i] || null;

            i++;
        }

        this.trace('spots in createView', this.spots);

        for (n = 1; n <= this.options.max_items; n++)
        {
            if (this.spots[n].item == null)
            {
                this.spots[n].elem.hide();
            }
            else
            {
                this.spots[n].img.attr('src', this.spots[n].item.thumbnail_url);

                if (n == mid_index)
                {
                    this.spots[n].elem.find('.js_scfi_title').html(this.spots[n].item.title);
                    this.spots[n].elem.find('.js_item_link').click(function(e)
                    {
                        e.preventDefault();
                        project.navigateTo('episodes.html?showid=' + that.spots[mid_index].item.id + '#subpage=latest_episodes');
                    });

                    var description = project.string_toolkit.createCleanExcerpt(this.spots[n].item.description, 40);
                    this.spots[n].elem.find('.js_scfi_description').html(description);
                }

                this.spots[n].elem.show();
            }
        }
    },

    rotateRight: function()
    {
        this.trace('rotateRight');

        if (this.shows[this.subpage].index < this.shows[this.subpage].items.length - 1)
        {
            this.shows[this.subpage].index += 1;
            this.createView(this.shows[this.subpage].items.concat([]));
        }
    },

    rotateLeft: function()
    {
        this.trace('rotateLeft');

        if (this.shows[this.subpage].index > 0)
        {
            this.shows[this.subpage].index += -1;
            this.createView(this.shows[this.subpage].items.concat([]));
        }
    },

    initializeNavigation: function()
    {
        var that = this;

        this.key_controller = new gtv.jq.KeyController();

        var nav_key_mapping = {};

        nav_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('#coverflow .js_item');
            if (!newSelected.length)
            {
                newSelected = jQuery('#subnav .js_item');
            }

            if (newSelected.length)
            {
                return new gtv.jq.Selection('selected', newSelected);
            }

            return new gtv.jq.Selection('none');
        };
        nav_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('#subnav .js_item');
            return new gtv.jq.Selection('selected', newSelected);
        };

        var subnav_key_mapping = {};

        subnav_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            selectedItem.click();
            selectedItem.click(); //workaround for samsung
            return new gtv.jq.Selection('none');
        };
        subnav_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('skip');
        };
        subnav_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            return new gtv.jq.Selection('skip');
        };
        subnav_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('#coverflow .js_item');
            return new gtv.jq.Selection('selected', newSelected);
        };

        var shows_key_mapping = {};

        shows_key_mapping[project.keycodes.KEY_ENTER] = function(selectedItem, newSelected)
        {
            selectedItem.click();
            return new gtv.jq.Selection('none');
        };
        shows_key_mapping[project.keycodes.KEY_LEFT] = function(selectedItem, newSelected)
        {
            that.rotateLeft();
            return new gtv.jq.Selection('skip');
        };
        shows_key_mapping[project.keycodes.KEY_UP] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('#subnav .js_item');
            return new gtv.jq.Selection('selected', newSelected);
        };
        shows_key_mapping[project.keycodes.KEY_RIGHT] = function(selectedItem, newSelected)
        {
            that.rotateRight();
            return new gtv.jq.Selection('skip');
        };
        shows_key_mapping[project.keycodes.KEY_DOWN] = function(selectedItem, newSelected)
        {
            newSelected = jQuery('#nav').find('.js_item').first();
            return new gtv.jq.Selection('selected', newSelected);
        };

        this.nav_zone = project.getMenuBehaviourZone(nav_key_mapping);
        this.subnav_zone = project.getSubMenuBehaviourZone(subnav_key_mapping);
        this.shows_zone = project.getDefaultBehaviourZone('#coverflow', shows_key_mapping);

        this.key_controller.addBehaviorZone(this.nav_zone);
        this.key_controller.addBehaviorZone(this.subnav_zone);
        this.key_controller.addBehaviorZone(this.shows_zone);

        this.key_controller.start(this.nav_zone, true);
    }
};

jQuery.extend(true, project.ShowsOverview.prototype, new project.Log());

JsBehaviourToolkit.registerHandler('shows_overview', project.ShowsOverview);
