project.StringToolkit = function(options)
{
    if (options && typeof options.default_excerpt_length !== 'undefined')
    {
        this.default_excerpt_length = options.default_excerpt_length || 190;
    }

    if (options && typeof options.default_excerpt_append_char !== 'undefined')
    {
        this.default_excerpt_append_char = options.default_excerpt_append_char || '';
    }

    if (options && typeof options.default_excerpt_simple_mode !== 'undefined')
    {
        this.default_excerpt_simple_mode = options.default_excerpt_simple_mode || false;
    }
};

project.StringToolkit.prototype = {
    /**
     * Creates an excerpt of given text with given length. Simple mode just
     * cuts the text after length chars whileas by default the appendChar is
     * appended after the last token (tab, newline, space) found.
     */
    createExcerpt: function(text, length, custom_append_char, use_simple_mode)
    {
        if (!text || typeof(text) != 'string')
        {
            return '';
        }

        var text_len = text.length;
        var len = +length || this.default_excerpt_length || false;
        if (
            typeof(length) == 'undefined' ||
            false === len ||
            ((typeof(len) == 'number') && (parseInt(len, 10) == len) && len <= 0)
        )
        {
            return text;
        }

        var append_char = custom_append_char || this.default_excerpt_append_char;
        var simple_mode = use_simple_mode || this.default_excerpt_simple_mode;

        if (text_len <= len)
        {
           return text;
        }

        var excerpt = '';

        if (true === simple_mode)
        {
            text = text.substr(0, len);
            return text + append_char;
        }

        var token = this.strtok(text, " \n\t");

        var current_length = 0;

        var token_length = token.length;

        while ((false !== token) && ((current_length + token_length + 1) < length))
        {
            excerpt += ' '+token;
            current_length += token_length + 1;
            token = this.strtok(" \n\t");
            token_length = token.length;
        }
        
        return excerpt + append_char;
    },

    /**
     * Tokenizes the given string.
     *
     * @see http://www.bloggapedia.com/blog_post.php?p=Javascript-equivalent-for-b-PHP-b-s-strtok-807792
     * @example string = "\t\t\t\nThis is\tan example\nstring\n"; tok = strtok($string, " \n\t");  $b = ''; while ($tok !== false) {$b += "Word="+$tok+"\n"; $tok = strtok(" \n\t");} => returns "Word=This\nWord=is\nWord=an\nWord=example\nWord=string\n"
     *
     * @param text string text to tokenize
     * @param tokens string token to use as separator
     *
     * @return false on empty string; tokenized string otherwise
     */
    strtok: function (text, tokens)
    {
        if (tokens === undefined)
        {
            tokens = text;
            text = this.strtokleftOver;
        }

        if (text.length === 0)
        {
            return false;
        }

        if (tokens.indexOf(text.charAt(0)) !== -1)
        {
            return this.strtok(text.substr(1), tokens);
        }
        
        for (var i=0; i < text.length; i++)
        {
            if (tokens.indexOf(text.charAt(i)) !== -1)
            {
                break;
            }
        }

        this.strtokleftOver = text.substr(i+1);

        return text.substring(0, i);
    },

    str_replace: function (haystack, needle, replacement)
    {
        var temp = haystack.split(needle);
        return temp.join(replacement);
    },

    str_replace_regex: function (haystack, needle, replacement)
    {
        var r = new RegExp(needle, 'g');
        return this.str_replace(haystack, r, replacement);
    },

    trim: function (str, chars)
    {
        return this.ltrim(this.rtrim(str, chars), chars);
    },

    ltrim: function (str, chars)
    {
        chars = chars || "\\s";
        return this.str_replace_regex(str, "^[" + chars + "]+", "");
    },

    rtrim: function (str, chars)
    {
        chars = chars || "\\s";
        return this.str_replace_regex(str, "[" + chars + "]+$", "");
    },

    /**
     * Strips HTML and PHP tags from a string.
     * @see http://phpjs.org/functions/strip_tags (version 1107.2516)
     */
    strip_tags: function (input, allowed)
    {
        allowed = (((allowed || "") + "").toLowerCase().match(/<[a-z][a-z0-9]*>/g) || []).join(''); // making sure the allowed arg is a string containing only tags in lowercase (<a><b><c>)
        var tags = /<\/?([a-z][a-z0-9]*)\b[^>]*>/gi,
            commentsAndPhpTags = /<!--[\s\S]*?-->|<\?(?:php)?[\s\S]*?\?>/gi;
        return input.replace(commentsAndPhpTags, '').replace(tags, function ($0, $1) {
            return allowed.indexOf('<' + $1.toLowerCase() + '>') > -1 ? $0 : '';
        });
    },

    clearString: function(string)
    {
        string = this.strip_tags(string);
        string = string.replace(/\/n/g, " ").replace(/&nbsp;/g, "");
        return this.trim(string);
    },

    createCleanExcerpt: function(text, length, custom_append_char, use_simple_mode)
    {
        text = this.clearString(text);
        return this.createExcerpt(text, length, custom_append_char, use_simple_mode);
    }
};

project.string_toolkit = new project.StringToolkit({
    'default_excerpt_append_char': '...'
});